#include "dsofinder.h"
#include "ui_dsofinder.h"
#include "QDesktopWidget"

DsoFinder::DsoFinder(int argc,char* argv[],QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::DsoFinder)
{
    //SET FLAGS FOR NON-DEFAULT WINDOW (NO MINIMIZE & MAXIMIZE BUTTON) AND TITLE
    Qt::WindowFlags flags = Qt::CustomizeWindowHint|Qt::WindowTitleHint|Qt::WindowCloseButtonHint;
    setWindowFlags(flags);
    ui->setupUi(this);
    this->setWindowTitle("DSO Finder build 13-11-23");

    //BEGIN DEBUG
    debug=FALSE;
    if(argc>1) if(strcmp(argv[1],"-d")==0) debug=TRUE;
    if(debug) {
        ui->notelabel->hide();
        ui->debugloadButton->show();
        ui->debugsaveButton->show();
    } else {
        ui->notelabel->show();
        ui->debugloadButton->hide();
        ui->debugsaveButton->hide();
        ui->fixlabel2->hide();
        ui->timelabel->hide();
    }
    //END DEBUG

    //START RESIZED SMALL
    ui->screenlabel->setText("");
    ui->screenlabel->hide();
    ui->findButton->hide();
    this->resize(100,50);

    //INIT VARS
    d_width = 0;
    d_heigth = 0;
    init_items(); //LOOK AT items.cpp FOR LIST OF ITEMS
}

DsoFinder::~DsoFinder()
{
    //DECONSTRUCTUR - KILLS CLASS
    delete ui;
}

void DsoFinder::on_takeButton_clicked()
{
    //TAKE SCREENSHOT AND ALLOCATE PROPPER PIXELARRAY
    this->resize(100,50);
    originalPixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
    if(originalPixmap.width() != d_width || originalPixmap.height() != d_heigth) { //ONLY REALLOCATE IF SCREENSIZE HAS CHANGED (MULTIMONITOR) -> DELETE MISSING!
        d_width = originalPixmap.width();
        d_heigth = originalPixmap.height();
        pixels = new int**[originalPixmap.width()];
        for(i=0;i<originalPixmap.width();i++) {
            pixels[i] = new int*[originalPixmap.height()];
            for(k=0;k<originalPixmap.height();k++) {
                pixels[i][k] = new int[3];
                for(j=0;j<3;j++) {
                    pixels[i][k][j] = 0;
                }
            }
        }
    }
    //RESIZE TO FULL AND SHOW SCREENSHOT
    originalImage = originalPixmap.toImage();
    ui->screenlabel->setPixmap(originalPixmap.scaled(ui->screenlabel->size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
    ui->screenlabel->show();
    this->resize(660,560);
    ui->findButton->show();

}

void DsoFinder::on_resizeButton_clicked()
{
    //RESIZE TO SMALL
    this->resize(100,50);
    ui->findButton->hide();
}

void DsoFinder::on_debugsaveButton_clicked()
{
    //DEBUGMODE: SAVE SCREENSHOT
    originalImage.save("screen.png",0,100);
}

void DsoFinder::draw_discovery(QPoint coords, QColor color, Qt::PenStyle style)
{
    //ON DISCOVERY PAINT CIRCLE OF GIVEN COLOR AND STYLE
    totaldiscoverys++;
    pen_dev.setColor(color);
    pen_dev.setStyle(style);
    paint_dev.setPen(pen_dev);
    paint_dev.drawEllipse(coords,20,20);
}

void DsoFinder::on_findButton_clicked()
{
    timer.start();  //START TIMER

    //OPEN FILE AND RESET PIXELARRAY
    results=fopen("results.dat","w");
    totaldiscoverys = 0;

    for(x=0;x<originalPixmap.width();x++) {
        for(y=0;y<originalPixmap.height();y++) {
            for(j=0;j<3;j++) {
                pixels[x][y][j] = 0;
            }
        }
    }

    //SCAN SCREENSHOT AND FILL PIXELARRAY
    //ACTUALLY ITS FASTER TO WRITE ALL PIXELS INTO AN INT-ARRAY FIRST THEN DIRECTLY ACCESS THE QImage-CLASS PIXELVALUE IN FURTHER CALCULATIONS. MIGHT BE COMPILER DEPENDEND.
    for(x=0;x<originalPixmap.width();x++) {
        for(y=0;y<originalPixmap.height();y++) {
            imgrgb = originalImage.pixel(x,y);
            r = qRed(imgrgb);
            g = qGreen(imgrgb);
            b = qBlue(imgrgb);
            pixels[x][y][0] = r;
            pixels[x][y][1] = g;
            pixels[x][y][2] = b;
        }
    }

    //INIT PAINT-DEVICE
    paint_dev.begin(&originalPixmap);
    pen_dev.setColor(QColor(255,0,0));
    pen_dev.setWidth(5);
    paint_dev.setPen(pen_dev);

    //MAIN SEARCH ALGORITHM
    for(x=borderbox;x<originalPixmap.width()-borderbox;x++) {       //SCAN IMAGE (X)
        for(y=borderbox;y<originalPixmap.height()-borderbox;y++) {  //SCAN IMAGE (Y)
            for(i=0;i<itemcount;i++) {                              //SCAN FOR ALL DEFINED ITEMS (items.cpp)
                item_disc = 0;
                item_negating = 0;
                for(k=0;k<items[i].pixelcount;k++) {                //SCAN ALL DEFINED PIXELS OF ITEMS
                    if(items[i].pixelcount - items[i].needed + item_disc >= k) { //SPEED UP IF DISCOVERY IMPOSSIBLE
                        if( pixels[x+items[i].pixel[k].pos.x_pos][y+items[i].pixel[k].pos.y_pos][0]>items[i].pixel[k].r.min &&
                            pixels[x+items[i].pixel[k].pos.x_pos][y+items[i].pixel[k].pos.y_pos][0]<items[i].pixel[k].r.max &&
                            pixels[x+items[i].pixel[k].pos.x_pos][y+items[i].pixel[k].pos.y_pos][1]>items[i].pixel[k].g.min &&
                            pixels[x+items[i].pixel[k].pos.x_pos][y+items[i].pixel[k].pos.y_pos][1]<items[i].pixel[k].g.max &&
                            pixels[x+items[i].pixel[k].pos.x_pos][y+items[i].pixel[k].pos.y_pos][2]>items[i].pixel[k].b.min &&
                            pixels[x+items[i].pixel[k].pos.x_pos][y+items[i].pixel[k].pos.y_pos][2]<items[i].pixel[k].b.max) {
                            if(!items[i].pixel[k].negating) item_disc++;
                            else item_negating++;
                        }
                    }
                }
                if(item_disc >= items[i].needed && item_negating==0) {             //IF ITEM IS DISCOVERED MARK IT ON IMAGE AND WRITE LOGFILE
                    fprintf(results,"Possible %s found at %i %i\n",items[i].caption,x,y);
                    draw_discovery(QPoint(x,y),items[i].color,items[i].penstyle);
                }
            }
        }
    }
    paint_dev.end();

    //SHOW RESULTING IMAGE AND COUNT
    ui->screenlabel->setPixmap(originalPixmap.scaled(ui->screenlabel->size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
    ui->countlabel->setText(QString("%1").arg(totaldiscoverys));
    if(debug) ui->timelabel->setText(QString("%1").arg(timer.elapsed()));   //SHOW CALCULATION TIME IF IN DEBUG MODE. HELPFULL FOR CODE-OPTIMIZATION AND COMPILER COMPARISON.
    fclose(results);
}

void DsoFinder::on_debugloadButton_clicked()
{
    //DEBUGMODE: LOAD SCREEN
    originalImage.load("screen.png");
    originalPixmap.load("screen.png");
    ui->screenlabel->setPixmap(originalPixmap.scaled(ui->screenlabel->size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
}

void DsoFinder::on_ontopcheckbox_clicked()
{
    //ENABLE/DISABLE ALLWAYS-ON-TOP
    if(ui->ontopcheckbox->isChecked()) setWindowFlags(Qt::WindowStaysOnTopHint);
    else setWindowFlags(Qt::CustomizeWindowHint|Qt::WindowTitleHint|Qt::WindowCloseButtonHint);
    this->show();
}
